package de.fhdw.wtf.parser.testWithScanner;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Iterator;
import java.util.Vector;

import org.junit.Before;
import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.AttributeModifier;
import de.fhdw.wtf.common.ast.Constructor;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.ByNameState;
import de.fhdw.wtf.common.ast.type.ClassModifier;
import de.fhdw.wtf.common.ast.type.ClassModifierService;
import de.fhdw.wtf.common.ast.type.ClassModifierTransient;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.MapType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.RegularClassType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.exception.parser.AbstractParserException;
import de.fhdw.wtf.common.exception.parser.NoCurlyBracketCloseException;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.keywords.AbstractToken;
import de.fhdw.wtf.common.token.keywords.GroupToken;
import de.fhdw.wtf.common.token.keywords.VisitableToken;
import de.fhdw.wtf.common.token.symbols.BracketOpenToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketOpenToken;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;

/**
 * Test the scanning an parsing of different Types.
 */
public class TestTypes {
	
	private CurlyBracketOpenToken curlyBracketOpenToken;
	private BracketOpenToken bracketOpenToken;
	
	/**
	 * Creates the most important Tokens.
	 * 
	 * @throws Exception
	 */
	@Before
	public void setUp() throws Exception {
		this.bracketOpenToken = BracketOpenToken.create(Position.create("", 1, 1, 0));
		this.curlyBracketOpenToken = CurlyBracketOpenToken.create(Position.create("", 1, 1, 0));
	}
	
	/**
	 * group1 : group = [ class1 : class = {attribute : (int1:Integer, int2:Integer, sum1:{Integer, String, Test} ,
	 * sum2:{Integer, Test});} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testProductSum1() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		final IdentifierToken testIdentifier = IdentifierToken.create("Test", Position.create("", 1, 1, 0));
		
		final String stringInput =
				"group1 : group = [ class1 : class = {attribute : (int1:Integer, int2:Integer, sum1:{Integer, String, Test} , sum2:{Integer, Test});} service transient;];";
		final SimpleScannerInput input = new SimpleScannerInput(stringInput);
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(stringIdentifier, stringByName);
		
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(testIdentifier));
		final TypeProxy typeProxyTest = TypeProxy.create(testIdentifier, testByName);
		
		final SumType sum1 = SumType.create(this.curlyBracketOpenToken);
		sum1.add(typeProxyInteger);
		sum1.add(typeProxyString);
		sum1.add(typeProxyTest);
		
		final SumType sum2 = SumType.create(this.curlyBracketOpenToken);
		sum2.add(typeProxyInteger);
		sum2.add(typeProxyTest);
		
		final ProductType product = ProductType.create(this.bracketOpenToken);
		final ProductElementType element1 =
				ProductElementType.create(
						"int1",
						typeProxyInteger,
						IdentifierToken.create("int1", Position.create("", 1, 1, 0)));
		final ProductElementType element2 =
				ProductElementType.create(
						"int2",
						typeProxyInteger,
						IdentifierToken.create("int2", Position.create("", 1, 1, 0)));
		final ProductElementType element3 =
				ProductElementType.create("sum1", sum1, IdentifierToken.create("sum1", Position.create("", 1, 1, 0)));
		final ProductElementType element4 =
				ProductElementType.create("sum2", sum2, IdentifierToken.create("sum2", Position.create("", 1, 1, 0)));
		product.addElement(element1);
		product.addElement(element2);
		product.addElement(element3);
		product.addElement(element4);
		
		final Attribute attribute =
				Attribute.create("attribute", product, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierService.create(VisitableToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(AbstractToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {attribute : (int1:Integer, str:String) ;} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testProductSum2() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {attribute : (int1:Integer, str:String) ;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(stringIdentifier, stringByName);
		
		final ProductType product = ProductType.create(this.bracketOpenToken);
		final ProductElementType element1 =
				ProductElementType.create(
						"int1",
						typeProxyInteger,
						IdentifierToken.create("int1", Position.create("", 1, 1, 0)));
		final ProductElementType element2 =
				ProductElementType.create(
						"str",
						typeProxyString,
						IdentifierToken.create("str", Position.create("", 1, 1, 0)));
		product.addElement(element1);
		product.addElement(element2);
		
		final Attribute attribute =
				Attribute.create("attribute", product, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierService.create(VisitableToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(AbstractToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {attribute : {Integer, String} ;} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testProductSum3() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {attribute : {Integer, String} ;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(stringIdentifier, stringByName);
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		sum.add(typeProxyInteger);
		sum.add(typeProxyString);
		
		final Attribute attribute =
				Attribute.create("attribute", sum, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierService.create(VisitableToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(AbstractToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {attribute : {(int1:Integer,int2:Integer,int42:Integer), String,
	 * (int3:Integer,int4:Integer), Test} ;} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testProductSum4() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		final IdentifierToken testIdentifier = IdentifierToken.create("Test", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {attribute : {(int1:Integer,int2:Integer,int42:Integer), String, (int3:Integer,int4:Integer), Test} ;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(stringIdentifier, stringByName);
		
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(testIdentifier));
		final TypeProxy typeProxyTest = TypeProxy.create(testIdentifier, testByName);
		
		final ProductType product1 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element11 =
				ProductElementType.create(
						"int1",
						typeProxyInteger,
						IdentifierToken.create("int1", Position.create("", 1, 1, 0)));
		final ProductElementType element12 =
				ProductElementType.create(
						"int2",
						typeProxyInteger,
						IdentifierToken.create("int2", Position.create("", 1, 1, 0)));
		final ProductElementType element13 =
				ProductElementType.create(
						"int42",
						typeProxyInteger,
						IdentifierToken.create("int42", Position.create("", 1, 1, 0)));
		product1.addElement(element11);
		product1.addElement(element12);
		product1.addElement(element13);
		
		final ProductType product2 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element21 =
				ProductElementType.create(
						"int3",
						typeProxyInteger,
						IdentifierToken.create("int3", Position.create("", 1, 1, 0)));
		final ProductElementType element22 =
				ProductElementType.create(
						"int4",
						typeProxyInteger,
						IdentifierToken.create("int4", Position.create("", 1, 1, 0)));
		product2.addElement(element21);
		product2.addElement(element22);
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		sum.add(product1);
		sum.add(typeProxyString);
		sum.add(product2);
		sum.add(typeProxyTest);
		
		final Attribute attribute =
				Attribute.create("attribute", sum, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierService.create(VisitableToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(AbstractToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {attribute : {(sum:{Integer, String}, int42:Integer), String,
	 * (int3:Integer,int4:Integer), Test} ;} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testProductSum5() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		final IdentifierToken testIdentifier = IdentifierToken.create("Test", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {attribute : {(sum:{Integer, String}, int42:Integer), String, (int3:Integer,int4:Integer), Test} ;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(stringIdentifier, stringByName);
		
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(testIdentifier));
		final TypeProxy typeProxyTest = TypeProxy.create(testIdentifier, testByName);
		
		final SumType sumP = SumType.create(this.curlyBracketOpenToken);
		sumP.add(typeProxyInteger);
		sumP.add(typeProxyString);
		
		final ProductType product1 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element11 =
				ProductElementType.create("sum", sumP, IdentifierToken.create("sum", Position.create("", 1, 1, 0)));
		final ProductElementType element12 =
				ProductElementType.create(
						"int42",
						typeProxyInteger,
						IdentifierToken.create("int42", Position.create("", 1, 1, 0)));
		product1.addElement(element11);
		product1.addElement(element12);
		
		final ProductType product2 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element21 =
				ProductElementType.create(
						"int3",
						typeProxyInteger,
						IdentifierToken.create("int3", Position.create("", 1, 1, 0)));
		final ProductElementType element22 =
				ProductElementType.create(
						"int4",
						typeProxyInteger,
						IdentifierToken.create("int4", Position.create("", 1, 1, 0)));
		product2.addElement(element21);
		product2.addElement(element22);
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		sum.add(product1);
		sum.add(typeProxyString);
		sum.add(product2);
		sum.add(typeProxyTest);
		
		final Attribute attribute =
				Attribute.create("attribute", sum, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierService.create(VisitableToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(AbstractToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {attribute : (sum1:{Integer, Test}, sum2:{{Integer, String}, (int3:Integer,
	 * int4:Integer), Test}) ;} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testProductSum6() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		final IdentifierToken testIdentifier = IdentifierToken.create("Test", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {attribute : (sum1:{Integer, Test}, sum3:{{Integer, String}, (int3:Integer, int4:Integer), Test}) ;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(stringIdentifier, stringByName);
		
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(testIdentifier));
		final TypeProxy typeProxyTest = TypeProxy.create(testIdentifier, testByName);
		
		final SumType sum1 = SumType.create(this.curlyBracketOpenToken);
		sum1.add(typeProxyInteger);
		sum1.add(typeProxyTest);
		
		final ProductType product1 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element11 =
				ProductElementType.create(
						"int3",
						typeProxyInteger,
						IdentifierToken.create("int3", Position.create("", 1, 1, 0)));
		final ProductElementType element12 =
				ProductElementType.create(
						"int4",
						typeProxyInteger,
						IdentifierToken.create("int4", Position.create("", 1, 1, 0)));
		product1.addElement(element11);
		product1.addElement(element12);
		
		final SumType sum3 = SumType.create(this.curlyBracketOpenToken);
		sum3.add(typeProxyInteger);
		sum3.add(typeProxyString);
		
		final SumType sum2 = SumType.create(this.curlyBracketOpenToken);
		sum2.add(sum3);
		sum2.add(product1);
		sum2.add(typeProxyTest);
		
		final ProductType product2 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element21 =
				ProductElementType.create("sum1", sum1, IdentifierToken.create("sum1", Position.create("", 1, 1, 0)));
		final ProductElementType element22 =
				ProductElementType.create("sum3", sum2, IdentifierToken.create("sum3", Position.create("", 1, 1, 0)));
		product2.addElement(element21);
		product2.addElement(element22);
		
		final Attribute attribute =
				Attribute.create("attribute", product2, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierService.create(VisitableToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(AbstractToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {attribute : (sum1:{Integer, Test}, sum2:{{Integer, (str1:String,
	 * str2:String)}, (int3:Integer, int4:Integer), Test}) ;} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testProductSum7() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		final IdentifierToken testIdentifier = IdentifierToken.create("Test", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {attribute : (sum1:{Integer, Test}, sum2:{{Integer, (str1:String, str2:String)}, (int3:Integer, int4:Integer), Test}) ;} service transient;];");
		
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(stringIdentifier, stringByName);
		
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(testIdentifier));
		final TypeProxy typeProxyTest = TypeProxy.create(testIdentifier, testByName);
		
		final SumType sum1 = SumType.create(IdentifierToken.create("sum1", Position.create("", 1, 1, 0)));
		sum1.add(typeProxyInteger);
		sum1.add(typeProxyTest);
		
		final ProductType product1 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element1 =
				ProductElementType.create(
						"str1",
						typeProxyString,
						IdentifierToken.create("str1", Position.create("", 1, 1, 0)));
		final ProductElementType element2 =
				ProductElementType.create(
						"str2",
						typeProxyString,
						IdentifierToken.create("str2", Position.create("", 1, 1, 0)));
		product1.addElement(element1);
		product1.addElement(element2);
		
		final SumType sum2 = SumType.create(this.curlyBracketOpenToken);
		sum2.add(typeProxyInteger);
		sum2.add(product1);
		
		final ProductType product2 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element3 =
				ProductElementType.create(
						"int3",
						typeProxyInteger,
						IdentifierToken.create("int3", Position.create("", 1, 1, 0)));
		final ProductElementType element4 =
				ProductElementType.create(
						"int4",
						typeProxyInteger,
						IdentifierToken.create("int4", Position.create("", 1, 1, 0)));
		product2.addElement(element3);
		product2.addElement(element4);
		
		final SumType sum3 = SumType.create(this.curlyBracketOpenToken);
		sum3.add(sum2);
		sum3.add(product2);
		sum3.add(typeProxyTest);
		
		final ProductType product3 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element5 =
				ProductElementType.create("sum1", sum1, IdentifierToken.create("sum1", Position.create("", 1, 1, 0)));
		final ProductElementType element6 =
				ProductElementType.create("sum2", sum3, IdentifierToken.create("sum2", Position.create("", 1, 1, 0)));
		product3.addElement(element5);
		product3.addElement(element6);
		
		final Attribute attribute =
				Attribute.create("attribute", product3, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierService.create(VisitableToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(AbstractToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
		
	}
	
	/**
	 * group1 : group = [ class1 : class = {attribute : (sum1:{Integer, String}, sum2:{{Integer, String}, (int3:Integer,
	 * sum4: {Integer, Test})}) ;} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testProductSum8() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		final IdentifierToken testIdentifier = IdentifierToken.create("Test", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {attribute : (sum1:{Integer, String}, sum2:{{Integer, String}, (int3:Integer, sum4: {Integer, Test})}) ;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(stringIdentifier, stringByName);
		
		final ByNameState testByName = ByNameState.create(UnqualifiedName.create(testIdentifier));
		final TypeProxy typeProxyTest = TypeProxy.create(testIdentifier, testByName);
		
		final SumType sum1 = SumType.create(this.curlyBracketOpenToken);
		sum1.add(typeProxyInteger);
		sum1.add(typeProxyString);
		
		final SumType sum4 = SumType.create(this.curlyBracketOpenToken);
		sum4.add(typeProxyInteger);
		sum4.add(typeProxyTest);
		
		final ProductType product1 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element11 =
				ProductElementType.create(
						"int3",
						typeProxyInteger,
						IdentifierToken.create("int3", Position.create("", 1, 1, 0)));
		final ProductElementType element12 =
				ProductElementType.create("sum4", sum4, IdentifierToken.create("sum4", Position.create("", 1, 1, 0)));
		product1.addElement(element11);
		product1.addElement(element12);
		
		final SumType sum2 = SumType.create(this.curlyBracketOpenToken);
		sum2.add(sum1);
		sum2.add(product1);
		
		final ProductType product2 = ProductType.create(this.bracketOpenToken);
		final ProductElementType element21 =
				ProductElementType.create("sum1", sum1, IdentifierToken.create("sum1", Position.create("", 1, 1, 0)));
		final ProductElementType element22 =
				ProductElementType.create("sum2", sum2, IdentifierToken.create("sum2", Position.create("", 1, 1, 0)));
		product2.addElement(element21);
		product2.addElement(element22);
		
		final Attribute attribute =
				Attribute.create("attribute", product2, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierService.create(VisitableToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(AbstractToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
	/**
	 * group1 : group = [ class1 : class = {att1 :
	 * (prod1:{Integer1,Integer2},prod2:{Integer42,String},prod3:{Integer4,Test) ;} service transient;];" .
	 */
	@Test
	public void testProductSum9() {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {attribute : (sum1:{Integer, String}, sum2:{Integer, String}, sum3:{Integer, Test) ;} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		boolean isRightException = false;
		try {
			parser.parse();
		} catch (final NoValidTokenStreamException e) {
			if (parser.getExceptions().size() != 1) {
				fail();
			}
			final Iterator<AbstractParserException> i = parser.getExceptions().iterator();
			while (i.hasNext()) {
				final AbstractParserException current = i.next();
				if (current instanceof NoCurlyBracketCloseException) {
					isRightException = true;
				}
			}
		}
		assertTrue(isRightException);
	}
	
	/**
	 * group1 : group = [ class1 : class = {attribute : [String->Integer];} service transient;]; .
	 * 
	 * @throws Exception
	 */
	@Test
	public void testMap() throws Exception {
		final IdentifierToken classIdentifier = IdentifierToken.create("class1", Position.create("", 1, 1, 0));
		final IdentifierToken groupIdentifier = IdentifierToken.create("group1", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("attribute", Position.create("", 1, 1, 0));
		final IdentifierToken integerIdentifier = IdentifierToken.create("Integer", Position.create("", 1, 1, 0));
		final IdentifierToken stringIdentifier = IdentifierToken.create("String", Position.create("", 1, 1, 0));
		
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"group1 : group = [ class1 : class = {attribute : [String->Integer];} service transient;];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		final Model model = parser.parse();
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		final Vector<ClassModifier> modifiers = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, GroupToken.create(Position.create("", 1, 1, 0)));
		final Name className = groupName.addName(classIdentifier);
		final ClassType clazz =
				RegularClassType.create(
						className,
						modifiers,
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(integerIdentifier, integerByName);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(stringIdentifier, stringByName);
		
		final MapType map = MapType.create(stringIdentifier, typeProxyString, typeProxyInteger);
		
		final Attribute attribute =
				Attribute.create("attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		modifiers.add(ClassModifierService.create(VisitableToken.create(Position.create("", 1, 1, 0))));
		modifiers.add(ClassModifierTransient.create(AbstractToken.create(Position.create("", 1, 1, 0))));
		groupVector.add(clazz);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		assertEquals(expected, model);
	}
	
}
